/*
 * Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies). 
 * All rights reserved.
 * This component and the accompanying materials are made available
 * under the terms of "Eclipse Public License v1.0"
 * which accompanies this distribution, and is available
 * at the URL "http://www.eclipse.org/legal/epl-v10.html".
 * 
 * Initial Contributors:
 * Nokia Corporation - Initial contribution
 * 
 * Contributors:
 * 
 * Description: This file is part of com.nokia.tools.variant.compare component.
 */

package com.nokia.tools.variant.compare.cmodel.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemColorProvider;
import org.eclipse.emf.edit.provider.IItemFontProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITableItemColorProvider;
import org.eclipse.emf.edit.provider.ITableItemFontProvider;
import org.eclipse.emf.edit.provider.ITableItemLabelProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import com.nokia.tools.variant.compare.cmodel.util.ECompareModelAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers.
 * The adapters generated by this factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets.
 * Note that most of the adapters are shared among multiple instances.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class ECompareModelItemProviderAdapterFactory extends ECompareModelAdapterFactory implements ComposeableAdapterFactory, IChangeNotifier, IDisposable {
	/**
	 * This keeps track of the root adapter factory that delegates to this adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ComposedAdapterFactory parentAdapterFactory;

	/**
	 * This is used to implement {@link org.eclipse.emf.edit.provider.IChangeNotifier}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IChangeNotifier changeNotifier = new ChangeNotifier();

	/**
	 * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected Collection<Object> supportedTypes = new ArrayList<Object>();

	/**
	 * This constructs an instance.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public ECompareModelItemProviderAdapterFactory() {
		supportedTypes.add(IEditingDomainItemProvider.class);
		supportedTypes.add(IStructuredItemContentProvider.class);
		supportedTypes.add(ITreeItemContentProvider.class);
		supportedTypes.add(IItemLabelProvider.class);
		supportedTypes.add(IItemPropertySource.class);
		supportedTypes.add(ITableItemLabelProvider.class);
		supportedTypes.add(ITableItemColorProvider.class);
		supportedTypes.add(ITableItemFontProvider.class);
		supportedTypes.add(IItemColorProvider.class);
		supportedTypes.add(IItemFontProvider.class);
		
		
		supportedTypes.add(IItemCheckEnabledProvider.class);
		supportedTypes.add(ITableItemTooltipProvider.class);
		supportedTypes.add(ITableItemAnnotationProvider.class);
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EModelRoot} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EModelRootItemProvider eModelRootItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EModelRoot}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEModelRootAdapter() {
		if (eModelRootItemProvider == null) {
			eModelRootItemProvider = new EModelRootItemProvider(this);
		}

		return eModelRootItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffGroup} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffGroupItemProvider eDiffGroupItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffGroup}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffGroupAdapter() {
		if (eDiffGroupItemProvider == null) {
			eDiffGroupItemProvider = new EDiffGroupItemProvider(this);
		}

		return eDiffGroupItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffSimple} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffSimpleItemProvider eDiffSimpleItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffSimple}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffSimpleAdapter() {
		if (eDiffSimpleItemProvider == null) {
			eDiffSimpleItemProvider = new EDiffSimpleItemProvider(this);
		}

		return eDiffSimpleItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffSequence} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffSequenceItemProvider eDiffSequenceItemProvider;

	private EDiffSequenceItemItemProvider eDiffSequenceItemItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffSequence}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffSequenceAdapter() {
		if (eDiffSequenceItemProvider == null) {
			eDiffSequenceItemProvider = new EDiffSequenceItemProvider(this);
		}

		return eDiffSequenceItemProvider;
	}
	
	public Adapter createEDiffSequenceItemAdapter() {
		if (eDiffSequenceItemItemProvider == null) {
			eDiffSequenceItemItemProvider = new EDiffSequenceItemItemProvider(this);
		}

		return eDiffSequenceItemItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffResourceSetting2} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffResourceSetting2ItemProvider eDiffResourceSetting2ItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffResourceSetting2}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffResourceSetting2Adapter() {
		if (eDiffResourceSetting2ItemProvider == null) {
			eDiffResourceSetting2ItemProvider = new EDiffResourceSetting2ItemProvider(this);
		}

		return eDiffResourceSetting2ItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffLocalPathSetting} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffLocalPathSettingItemProvider eDiffLocalPathSettingItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffLocalPathSetting}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffLocalPathSettingAdapter() {
		if (eDiffLocalPathSettingItemProvider == null) {
			eDiffLocalPathSettingItemProvider = new EDiffLocalPathSettingItemProvider(this);
		}

		return eDiffLocalPathSettingItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.EDiffMismatch} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EDiffMismatchItemProvider eDiffMismatchItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.EDiffMismatch}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEDiffMismatchAdapter() {
		if (eDiffMismatchItemProvider == null) {
			eDiffMismatchItemProvider = new EDiffMismatchItemProvider(this);
		}

		return eDiffMismatchItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link com.nokia.tools.variant.compare.cmodel.ECheckableElement} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ECheckableElementItemProvider eCheckableElementItemProvider;

	/**
	 * This creates an adapter for a {@link com.nokia.tools.variant.compare.cmodel.ECheckableElement}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createECheckableElementAdapter() {
		if (eCheckableElementItemProvider == null) {
			eCheckableElementItemProvider = new ECheckableElementItemProvider(this);
		}

		return eCheckableElementItemProvider;
	}

	/**
	 * This returns the root adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ComposeableAdapterFactory getRootAdapterFactory() {
		return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
	}

	/**
	 * This sets the composed adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory) {
		this.parentAdapterFactory = parentAdapterFactory;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object type) {
		
		return supportedTypes.contains(type) || super.isFactoryForType(type);
	}

	/**
	 * This implementation substitutes the factory itself as the key for the adapter.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter adapt(Notifier notifier, Object type) {
		return super.adapt(notifier, this);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object adapt(Object object, Object type) {
		if (isFactoryForType(type)) {
			Object adapter = super.adapt(object, type);
			if (!(type instanceof Class) || (((Class<?>)type).isInstance(adapter))) {
				return adapter;
			}
		}

		return null;
	}

	/**
	 * This adds a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void addListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.addListener(notifyChangedListener);
	}

	/**
	 * This removes a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void removeListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.removeListener(notifyChangedListener);
	}

	/**
	 * This delegates to {@link #changeNotifier} and to {@link #parentAdapterFactory}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void fireNotifyChanged(Notification notification) {
		changeNotifier.fireNotifyChanged(notification);

		if (parentAdapterFactory != null) {
			parentAdapterFactory.fireNotifyChanged(notification);
		}
	}

	/**
	 * This disposes all of the item providers created by this factory. 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void dispose() {
		if (eModelRootItemProvider != null) eModelRootItemProvider.dispose();
		if (eDiffGroupItemProvider != null) eDiffGroupItemProvider.dispose();
		if (eDiffSimpleItemProvider != null) eDiffSimpleItemProvider.dispose();
		if (eDiffSequenceItemProvider != null) eDiffSequenceItemProvider.dispose();
		if (eDiffResourceSetting2ItemProvider != null) eDiffResourceSetting2ItemProvider.dispose();
		if (eDiffLocalPathSettingItemProvider != null) eDiffLocalPathSettingItemProvider.dispose();
		if (eDiffMismatchItemProvider != null) eDiffMismatchItemProvider.dispose();
		if (eCheckableElementItemProvider != null) eCheckableElementItemProvider.dispose();
	}

}
